package com.mortals.xhx.module.company.service.impl;

import com.mortals.framework.model.PageInfo;
import com.mortals.xhx.module.company.model.*;
import com.mortals.xhx.module.company.service.CompanyPatentService;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.company.dao.CompanyDao;
import com.mortals.xhx.module.company.service.CompanyService;
import org.springframework.beans.factory.annotation.Autowired;
import com.mortals.xhx.module.company.service.CompanyLabelsService;
import org.springframework.util.ObjectUtils;

import java.util.Date;
import java.util.Arrays;
import java.util.List;

import lombok.extern.slf4j.Slf4j;

/**
 * CompanyService
 * 公司 service实现
 *
 * @author zxfei
 * @date 2023-09-18
 */
@Service("companyService")
@Slf4j
public class CompanyServiceImpl extends AbstractCRUDServiceImpl<CompanyDao, CompanyEntity, Long> implements CompanyService {
    @Autowired
    private CompanyLabelsService companyLabelsService;
    @Autowired
    private CompanyPatentService companyPatentService;

    @Override
    protected void findAfter(CompanyEntity params, PageInfo pageInfo, Context context, List<CompanyEntity> list) throws AppException {

        list.stream().peek(item -> {
            List<CompanyPatentEntity> companyPatentEntities = companyPatentService.find(new CompanyPatentQuery().companyId(item.getId()));
            item.setCompanyPatentList(companyPatentEntities);
        }).count();
    }

    @Override
    protected void saveAfter(CompanyEntity entity, Context context) throws AppException {
        if (!ObjectUtils.isEmpty(entity.getCompanyLabelsList())) {
            entity.getCompanyLabelsList().stream().peek(item -> {
                item.setCompanyId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
            }).count();
            companyLabelsService.save(entity.getCompanyLabelsList());
        }

        if (!ObjectUtils.isEmpty(entity.getCompanyPatentList())) {
            entity.getCompanyPatentList().stream().peek(item -> {
                item.setCompanyId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
            }).count();
            companyPatentService.save(entity.getCompanyPatentList());
        }
        super.saveAfter(entity, context);
    }

    @Override
    protected void updateAfter(CompanyEntity entity, Context context) throws AppException {
        if (!ObjectUtils.isEmpty(entity.getCompanyLabelsList())) {
            Long[] companyLabelsIds = companyLabelsService.find(new CompanyLabelsQuery().companyId(entity.getId())).stream().map(CompanyLabelsEntity::getId).toArray(Long[]::new);
            companyLabelsService.remove(companyLabelsIds, context);
            entity.getCompanyLabelsList().stream().peek(item -> {
                item.setCompanyId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
                item.setUpdateUserId(this.getContextUserId(context));
                item.setUpdateTime(new Date());
            }).count();
            companyLabelsService.save(entity.getCompanyLabelsList());
        }

        if (!ObjectUtils.isEmpty(entity.getCompanyPatentList())) {
            Long[] companyPantentIds = companyPatentService.find(new CompanyPatentQuery().companyId(entity.getId())).stream().map(CompanyPatentEntity::getId).toArray(Long[]::new);
            companyPatentService.remove(companyPantentIds, context);
            entity.getCompanyLabelsList().stream().peek(item -> {
                item.setCompanyId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
                item.setUpdateUserId(this.getContextUserId(context));
                item.setUpdateTime(new Date());
            }).count();
            companyPatentService.save(entity.getCompanyPatentList());
        }

        super.updateAfter(entity, context);
    }

    @Override
    protected void removeAfter(Long[] ids, Context context, int result) throws AppException {
        List<CompanyLabelsEntity> companyLabelslist = companyLabelsService.find(new CompanyLabelsQuery().companyIdList(Arrays.asList(ids)));
        companyLabelsService.removeList(companyLabelslist, context);

        List<CompanyPatentEntity> companyPatentEntities = companyPatentService.find(new CompanyPatentQuery().companyIdList(Arrays.asList(ids)));
        companyPatentService.removeList(companyPatentEntities, context);


        super.removeAfter(ids, context, result);
    }
}